//! Data models for sprinklers sections
//!
//! A section represents a group of sprinkler heads actuated by a single
//! valve. Physically controllable (or virtual) valves are handled by implementations of
//! [SectionInterface](../../section_interface/trait.SectionInterface.html), but the model
//! describes a logical section and how it maps to a physical one.

use crate::section_interface::SecId;
use rusqlite::{Error as SqlError, Row as SqlRow, ToSql};
use serde::{Deserialize, Serialize};
use std::sync::Arc;

/// Identifying integer type for a Section
pub type SectionId = u32;

/// A single logical section
#[derive(Debug, Clone, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Section {
    pub id: SectionId,
    pub name: String,
    /// ID number of the corresponding physical section
    pub interface_id: SecId,
}

impl Section {
    pub fn from_sql<'a>(row: &SqlRow<'a>) -> Result<Section, SqlError> {
        Ok(Section {
            id: row.get(0)?,
            name: row.get(1)?,
            interface_id: row.get(2)?,
        })
    }

    #[allow(dead_code)]
    pub fn as_sql(&self) -> Vec<&dyn ToSql> {
        vec![&self.id, &self.name, &self.interface_id]
    }
}

pub type SectionRef = Arc<Section>;

pub type Sections = im::OrdMap<SectionId, SectionRef>;
