use super::*;
use crate::{model::SectionRef, section_runner::SectionRunHandle};
use eyre::WrapErr;
use serde::{Deserialize, Serialize};
use std::time::Duration;

#[derive(Copy, Clone, Debug, Serialize, Deserialize)]
#[serde(transparent)]
pub struct SectionId(pub crate::model::SectionId);

impl SectionId {
    fn get_section(self, sections: &Sections) -> Result<SectionRef, RequestError> {
        sections.get(&self.0).cloned().ok_or_else(|| {
            RequestError::with_name(ErrorCode::NoSuchSection, "section not found", "section")
        })
    }
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct RunSectionRequest {
    pub section_id: SectionId,
    #[serde(with = "crate::serde::duration")]
    pub duration: Duration,
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct RunSectionResponse {
    pub message: String,
    pub run_id: SectionRunHandle,
}

impl IRequest for RunSectionRequest {
    type Response = RunSectionResponse;
    fn exec(&mut self, ctx: &mut RequestContext) -> RequestFuture<Self::Response> {
        let mut section_runner = ctx.section_runner.clone();
        let section = self.section_id.get_section(&ctx.sections);
        let duration = self.duration;
        Box::pin(async move {
            let section = section?;
            let handle = section_runner
                .queue_run(section.clone(), duration)
                .await
                .wrap_err("could not queue run")?;
            Ok(RunSectionResponse {
                message: format!("running section '{}' for {:?}", &section.name, duration),
                run_id: handle,
            })
        })
    }
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct CancelSectionRequest {
    pub section_id: SectionId,
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct CancelSectionResponse {
    pub message: String,
    pub cancelled: usize,
}

impl IRequest for CancelSectionRequest {
    type Response = CancelSectionResponse;
    fn exec(&mut self, ctx: &mut RequestContext) -> RequestFuture<Self::Response> {
        let mut section_runner = ctx.section_runner.clone();
        let section = self.section_id.get_section(&ctx.sections);
        Box::pin(async move {
            let section = section?;
            let cancelled = section_runner
                .cancel_by_section(section.id)
                .await
                .wrap_err("could not cancel section")?;
            Ok(CancelSectionResponse {
                message: format!(
                    "cancelled {} runs for section '{}'",
                    cancelled, section.name
                ),
                cancelled,
            })
        })
    }
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct CancelSectionRunIdRequest {
    pub run_id: SectionRunHandle,
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct CancelSectionRunIdResponse {
    pub message: String,
    pub cancelled: bool,
}

impl IRequest for CancelSectionRunIdRequest {
    type Response = ResponseMessage;
    fn exec(&mut self, ctx: &mut RequestContext) -> RequestFuture<Self::Response> {
        let mut section_runner = ctx.section_runner.clone();
        let run_id = self.run_id.clone();
        Box::pin(async move {
            let cancelled = section_runner
                .cancel_run(run_id)
                .await
                .wrap_err("could not cancel section run")?;
            if cancelled {
                Ok(ResponseMessage::new("cancelled section run"))
            } else {
                Err(RequestError::with_name(
                    ErrorCode::NoSuchSectionRun,
                    "section run does not exist",
                    "section run",
                ))
            }
        })
    }
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct PauseSectionRunnerRequest {
    pub paused: bool,
}

#[derive(Debug, Deserialize, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct PauseSectionRunnerResponse {
    pub message: String,
    pub paused: bool,
}

impl IRequest for PauseSectionRunnerRequest {
    type Response = PauseSectionRunnerResponse;
    fn exec(&mut self, ctx: &mut RequestContext) -> RequestFuture<Self::Response> {
        let mut section_runner = ctx.section_runner.clone();
        let paused = self.paused;
        Box::pin(async move {
            if paused {
                section_runner.pause().await
            } else {
                section_runner.unpause().await
            }
            .wrap_err("could not pause/unpause section runner")?;
            Ok(PauseSectionRunnerResponse {
                message: format!(
                    "{} section runner",
                    if paused { "paused" } else { "unpaused" }
                ),
                paused,
            })
        })
    }
}
